﻿/*
 * @class Delegate
 * @version 0.2
 * @author Francis Bourre 
 * @description Inspired from mx.utils.Delegate and Peter Hall's EventDelegate. Visit tweenpix.net for explanations or updates.
 */
import com.delfiweb.command.*;

class com.delfiweb.command.Delegate implements _Command
{
 	private var _f:Function;
 	private var _o:Object;
 	private var _a:Array;
 	private var _fProxy:Function;
 
 	/*
 	 * @description Wrapper for MM compatibility.
 	 * @usage : com.delfiweb.events.Delegate.create(this, onDoSomehting);
 	 */
 	public static function create(o:Object, f:Function) : Delegate
 	{
  		return new Delegate(o, f, arguments.splice(2));
  	}
 	
 	/**
 	 * @description Creates a functions wrapper for the original function so that it runs in the provided context.
 	 * @usage : var d:Delegate = new Delegate(this, onDoSomehting);
 	 * @param o Context in which to run the function.
 	 * @param f Function to run.
 	 */
 	public function Delegate(o:Object, f:Function)
 	{
  		_o = o;
  		_f = f;
  		_a = arguments.splice(2);
  		_buildProxy();
  	}
 
 	public function handleEvent(e) : Object
 	{
  		return _f.apply(_o, [e].concat(_a));
  	}
 	
 	/**
 	 * @description Get proxy reference.
 	 * @usage myMovieClip.onPress = d.getFunction();
 	 */
 	public function getFunction() : Function
 	{
  		return _fProxy;
  	}
 	
 	/**
 	 * @description Call function in context (Command polymorphism).
 	 * @usage d.execute();
 	 * @see com.delfiweb.command._Command
 	 */
 	public function execute() : Void
 	{
  		_fProxy();
  	}
 	
 	/**
 	 * @description Change arguments of called function.
 	 * @usage d.setArguments(a0, a1, ..., an);
 	 */
 	public function setArguments() : Void
 	{
  		if (arguments.length > 0)
  		{
   			_a = arguments;
   			_fProxy.a = _a;
   		}
  	}
 	
 	/**
 	 * @description Add arguments to called function.
 	 * @usage d.addArguments(a0, a1, ..., an);
 	 */
 	public function addArguments() : Void
 	{
  		if (arguments.length > 0)
  		{
   			_a = _a.concat(arguments);
   			_fProxy.a = _a;
   		}
  	}
 	
 	//
 	private function _buildProxy() : Void
 	{	// MM original implementation updated
  		_fProxy = function()
  		{
   			var t = arguments.callee.t;
   			var f = arguments.callee.f;
   			var a = arguments.concat(arguments.callee.a);
   
   			return f.apply(t, a);
   		}
  
  		_fProxy.t = _o;
  		_fProxy.f = _f;
  		_fProxy.a = _a;
  	}
}
